package cz.kebrt.unitcheck.jpf;

import gov.nasa.jpf.PropertyListenerAdapter;
import gov.nasa.jpf.jvm.JVM;
import gov.nasa.jpf.search.Search;

/**
 * JPF property that allows to stop JPF execution. The property holds after
 * calling the <code>stop</code> method.
 * @author michal
 */
public class StopExecutionProperty extends PropertyListenerAdapter {

    public static final String USER_ABORT
        = "JPF execution stopped by the user.";
    public static final String EXCEPTION_CAUGHT_BY_JUNIT
        = "Exception caught in JUnit; JPF stopped.";

    private boolean stopped = false;
    private String message = null;

    @Override
    public boolean check(Search search, JVM vm) {
        return !stopped;
    }

    @Override
    public String getErrorMessage() {
        return message;
    }

    @Override
    public void reset() {
        stopped = false;
    }

    /**
     * Activates the property.
     * @param msg message that will be reported by JPF
     */
    public void stop(String msg) {
        this.message = msg;
        stopped = true;
    }

}
