package cz.kebrt.unitcheck.junit;

import org.junit.runner.Description;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunListener;

import cz.kebrt.unitcheck.jpf.JPF_cz_kebrt_unitcheck_junit_TestReportListener;
import cz.kebrt.unitcheck.junit.holder.ThrowableHolder;

/**
 * Collects information about the execution of JUnit tests. For each method, a
 * corresponding native method is provided to enable the usage of MJI (
 * {@link JPF_cz_kebrt_unitcheck_junit_TestReportListener} is the corresponding peer
 * class). The dual methods do not completely match their counterparts - they have
 * arguments of atomic types (integers, strings, ...) to make the access of information
 * provided by JUnit easier in the MJI peer class.
 *
 * @author michal
 */
public class TestReportListener extends RunListener {

    @Override
    public void testRunStarted(Description description) {
        testRunStartedNative(description.getDisplayName());
    }

    @Override
    public void testRunFinished(Result result) {
        testRunFinishedNative(result.wasSuccessful(), result.getFailureCount(), result
                .getIgnoreCount(), result.getRunCount(), result.getRunTime());
    }

    @Override
    public void testStarted(Description description) {
        testStartedNative(description.getDisplayName());
    }

    @Override
    public void testFinished(Description description) {
        testFinishedNative(description.getDisplayName());
    }

    @Override
    public void testFailure(Failure failure) {
        testFailureNative(new ThrowableHolder(failure.getException()), failure
                .getMessage(), failure.getTestHeader(), failure.getTrace());
    }

    @Override
    public void testAssumptionFailure(Failure failure) {
        testAssumptionFailureNative(new ThrowableHolder(failure.getException()), failure
                .getMessage(), failure.getTestHeader(), failure.getTrace());
    }

    @Override
    public void testIgnored(Description description) {
        testIgnoredNative(description.getDisplayName());
    }

    public native void testRunFinishedNative(boolean wasSuccessful, int failureCount,
            int ignoreCount, int runCount, long runTime);

    public native void testRunStartedNative(String description);

    public native void testStartedNative(String description);

    public native void testFinishedNative(String description);

    public native void testFailureNative(ThrowableHolder throwable, String message,
            String testHeader, String trace);

    public native void testAssumptionFailureNative(ThrowableHolder throwable,
            String message, String testHeader, String trace);

    public native void testIgnoredNative(String description);

}
