package cz.kebrt.unitcheck.config;

/**
 * Configuration of a test method/class to be executed.
 *
 * @author michal
 */
public class TestConfig {

    private Class<?> testClass;
    private String testMethod = null;

    /**
     * Constructs an instance.
     * @param testClass fully qualified name of a test class
     *
     * @throws ConfigException class could not be found
     */
    public TestConfig(String testClass) throws ConfigException {
        this(testClass, null);
    }

    /**
     * Constructs an instance.
     * @param testClass fully qualified name of a test class
     * @param testMethod name of a test method within the class
     *
     * @throws ConfigException class could not be found
     */
    public TestConfig(String testClass, String testMethod) throws ConfigException {
        Class<?> klass = null;
        try {
            klass = Class.forName(testClass, true, Thread.currentThread()
                    .getContextClassLoader());
        } catch (ClassNotFoundException e) {
            throw new ConfigException(String.format("Test class %s not found",
                    testClass));
        }
        this.testClass = klass;
        this.testMethod = testMethod;
    }

    /**
     * Returns name of the test class.
     *
     * @return name of the test class
     */
    public Class<?> getTestClass() {
        return testClass;
    }

    /**
     * Returns name of the test method.
     *
     * @return name of the test method (may be <code>null</code>)
     */
    public String getTestMethod() {
        return testMethod;
    }

}
