package cz.kebrt.unitcheck.test;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

//
//Copyright (C) 2006 United States Government as represented by the
//Administrator of the National Aeronautics and Space Administration
//(NASA).  All Rights Reserved.
//
//This software is distributed under the NASA Open Source Agreement
//(NOSA), version 1.3.  The NOSA has been approved by the Open Source
//Initiative.  See the file NOSA-1.3-JPF at the top of the distribution
//directory tree for the complete NOSA document.
//
//THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF ANY
//KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT
//LIMITED TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO
//SPECIFICATIONS, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//A PARTICULAR PURPOSE, OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT
//THE SUBJECT SOFTWARE WILL BE ERROR FREE, OR ANY WARRANTY THAT
//DOCUMENTATION, IF PROVIDED, WILL CONFORM TO THE SUBJECT SOFTWARE.
//
public class Peterson {
    int ncrit = 0;
    int numIds;
    int[] turn;
    int[] flag;

    boolean output;

    public void init(int ids, boolean output) {
        this.numIds = ids;
        this.output = output;
        turn = new int[numIds];
        flag = new int[numIds];
    }

    class Proc implements Runnable {
        Proc(int id) {
            this.id = id;
        }

        final int id;

        public void run() {
            if (output)
                System.out.println("Process " + id + " started");
            for (;;) {
                for (int k = 0; k < numIds - 1; k++) {
                    flag[id] = k;
                    turn[k] = id;
                    for (int j = 0; j < numIds; j++) {
                        if (j == id)
                            continue;
                        while (!(flag[j] < k || turn[k] != id))
                            Thread.yield();
                    }
                }

                ncrit++;
                if (ncrit != 1) {
                    throw new IllegalStateException();
                }
                if (output)
                    System.out.println("Process " + id + " in critical section");
                ncrit--;

                flag[id] = 0;
            }
        }
    }

    void go() {
        List<Thread> threads = new ArrayList<Thread>();
        for (int i = 0; i < numIds; i++) {
            Thread t = new Thread(new Proc(i));
            threads.add(t);
            t.start();
        }
        for (Thread t : threads) {
            try {
                t.join();
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }

    @Test
    public void main() {
        init(3, true);
        go();
    }
}
